<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\ClientInput;
use App\Dto\Output\ClientOutput;
use App\Dto\Output\UserGroupOutput;
use App\Repository\ClientRepository;
use App\Repository\MenuRepository;

readonly class ClientProcessor implements ProcessorInterface
{
    public function __construct(
        private ClientRepository   $clientRepository,
        private MenuRepository     $menuRepository,
        private ValidatorInterface $validator
    )
    {
    }

    /**
     * @throws \Exception
     */
    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = []): ClientOutput|null
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
            case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    /**
     * @throws \Exception
     */
    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): ClientOutput
    {
        if (!($data instanceof ClientInput)) {
            throw new \Exception(sprintf('data is not instance of %s', ClientInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->clientRepository->findOneByUuid($uriVariables['uuid']);
        }

        $defaultMenu = $this->menuRepository->findOneBy(['isDefault' => true]);

        $client = $data->createOrUpdateEntity($entity);

        if ($defaultMenu) {
            $client->setMenu($defaultMenu);
        }

        $this->validator->validate($client);
        $this->clientRepository->save($client);

        return new ClientOutput($client);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->clientRepository->findOneByUuid($uriVariables['uuid']);
        $this->clientRepository->delete($user);

        return null;
    }
}
